
################################################################################
##
## check to make SOURCES and TARGET is properly set
##
################################################################################

ifndef SOURCES
$(error SOURCES variable must be defined to be the sources you want to include in the build.)
endif

ifndef TARGET
$(error TARGET must be defined to be the name of the module to build.)
endif


################################################################################
##
## make sure CONTINUITY is properly configured
##
################################################################################

ifndef CONTINUITY
$(error CONTINUITY must be defined to point to the root of the continuity build.)
endif

ifeq ($shell(test -d $(CONTINUITY)/continuity/lib && echo found),found)
CONTINUITY := $(CONTINUITY)/continuity
endif

include $(CONTINUITY)/lib/env.mk

################################################################################
##
## Targets
##
################################################################################

ifndef TARGETTYPE
TARGETTYPE = module
endif

ifneq (,$(findstring module,$(TARGETTYPE)))
SHARED_TARGET = mod$(TARGET).so
ALL_TARGETS += $(SHARED_TARGET)
endif

ifneq (,$(findstring shared,$(TARGETTYPE)))
SHARED_TARGET = lib$(TARGET).so
ALL_TARGETS += $(SHARED_TARGET)
endif

ifneq (,$(findstring static,$(TARGETTYPE)))
STATIC_TARGET = lib$(TARGET).a
ALL_TARGETS += $(STATIC_TARGET)
endif

ifneq (,$(findstring executable,$(TARGETTYPE)))
EXECUTABLE_TARGET = $(TARGET)
ALL_TARGETS += $(EXECUTABLE_TARGET)
endif

################################################################################
##
## Dependancy mapping
##
################################################################################

DEPDIR = .deps
df = $(*D)/$(DEPDIR)/$(*F)

DEPENDFLAGS = -MD -MP -MF $(df).d

MAKE_DEP_DIR = [ -d "$(*D)$(DEPDIR)" ] || mkdir -p "$(*D)/$(DEPDIR)"

DEP_FILES = $(foreach dep,$(SOURCES),$(dir $(dep))$(DEPDIR)/$(notdir $(dep:%.c=%.d)))

################################################################################
##
## Cleare defaults we don't want
##
################################################################################

.SUFFIXES: 
.SUFFIXES: .o .c .h

.a:
%.a:


################################################################################
##
## setup variables
##
################################################################################

OBJECTS = $(SOURCES:%.c=%.o)

CFLAGS += -I$(CONTINUITY)/include -fPIC
CFLAGS += $(INCLUDES) $(LOCAL_CFLAGS) $(OPT_CFLAGS)

LIBS += $(LOCAL_LIBS)

# 
# i decided this isn't really needed
#  -eric 3.11.2004
#
# EXPANDED_STATIC_LIBS = $(STATIC_LIBS:%=-Wl,--whole-archive % -Wl,--no-whole-archive)

EXPANDED_STATIC_LIBS = $(STATIC_LIBS) $(WHOLE_STATIC_LIBS:%=-Wl,--whole-archive % -Wl,--no-whole-archive)


MAKEFILES = $(CONTINUITY)/lib/env.mk $(CONTINUITY)/lib/build.mk Makefile


################################################################################
##
## define targets
##
################################################################################

all: $(ALL_TARGETS) install

$(EXECUTABLE_TARGET): $(OBJECTS) $(STATIC_LIBS) $(MAKEFILES) $(WHOLE_STATIC_LIBS)
	$(CC) $(LD_FLAGS) $(OBJECTS) $(EXPANDED_STATIC_LIBS) $(LIBS) -o $@

$(SHARED_TARGET): $(OBJECTS) $(STATIC_LIBS) $(MAKEFILES) $(WHOLE_STATIC_LIBS)
	$(LD_SHARECMD) $(LD_FLAGS) $(PRE_BUILD) $(OBJECTS) $(EXPANDED_STATIC_LIBS) $(LIBS) $(POST_BUILD) -o $@

$(STATIC_TARGET): $(OBJECTS) $(MAKEFILES)
	$(RM) $@
	$(AR) crs $@ $(OBJECTS)

%.o: %.c $(MAKEFILES)
	@$(MAKE_DEP_DIR)
	$(CC) $(DEPENDFLAGS) $(CFLAGS) -c $< -o $@

ifneq ($(SHARED_TARGET),)
$(CONTINUITY)/lib/$(SHARED_TARGET): $(SHARED_TARGET)
	@echo "$< -> $@"
	@$(RM) $@
	@cp $< $@
endif

ifneq ($(STATIC_TARGET),)
$(CONTINUITY)/lib/$(STATIC_TARGET): $(STATIC_TARGET)
	@echo "$< -> $@"
	@$(RM) $@
	@cp $< $@
endif

ifneq ($(EXECUTABLE_TARGET),)
$(CONTINUITY)/bin/$(EXECUTABLE_TARGET): $(EXECUTABLE_TARGET)
	@echo "$< -> $@"
	@$(RM) $@
	@cp $< $@
endif

install: $(CONTINUITY)/lib/$(SHARED_TARGET)

#install: $(ALL_TARGETS)
#	@for file in $(ALL_TARGETS); do \
#		echo "$$file -> $(CONTINUITY)/lib/$$file"; \
#		$(RM) $(CONTINUITY)/lib/$$file; \
#		cp $$file $(CONTINUITY)/lib/$$file; \
#         done    

clean:
	$(RM) $(OBJECTS) $(DEP_FILES)

#ifneq ($(MAKECMDGOALS),clean)
#-include $(DEP_FILES)
#endif
