package lc.esp.engine.service;

import lc.mecha.json.JSONObject;
import lc.mecha.log.MechaLogger;
import lc.mecha.log.MechaLoggerFactory;
import org.apache.activemq.broker.Broker;
import org.apache.activemq.broker.BrokerFilter;
import org.apache.activemq.broker.ProducerBrokerExchange;
import org.apache.activemq.command.ActiveMQBytesMessage;
import org.apache.activemq.command.Message;
import org.apache.activemq.util.ByteSequence;

import java.io.IOException;
import java.nio.charset.StandardCharsets;

/**
 * This class implements the Enhanced Services Platform mechanics for ActiveMQ.
 *
 * @author Alex Leigh
 * @since mk18 (GIPSY DANGER)
 */
public class EnhancedBroker extends BrokerFilter {
    public static final String KEY_TYPE = "type";
    public static final String KEY_PARAMETERS = "parameters";
    public static final String KEY_PAYLOAD = "payload";
    public static final String V18_TYPE = "lc.esp.api18.envelope";
    private static final MechaLogger logger = MechaLoggerFactory.getLogger(EnhancedBroker.class);

    public EnhancedBroker(Broker next) {
        super(next);
        logger.info("Created new Enhanced Broker [broker: {}]", next);
    }

    public void send(ProducerBrokerExchange producerExchange, Message msg) throws Exception {
        logger.trace("send(): {}", msg);

        // For the time being all messages in ESP are JSON we can let this ride
        if (msg instanceof ActiveMQBytesMessage) {
            String s = new String(msg.getContent().getData());
            JSONObject envelope = new JSONObject(s);
            String type = envelope.optString(KEY_TYPE);
            logger.trace("Got type: {}", type);
            if (type != null) {
                if (type.equals(V18_TYPE)) {
                    JSONObject tags = envelope.optJSONObject(KEY_PARAMETERS);
                    if (tags != null) {
                        for (String key : tags.keySet()) {
                            try {
                                msg.setProperty(key, tags.get(key));
                            } catch (IOException e) {
                                logger.warn("Unable to set property: {}", e);
                            }
                        }
                    }
                    JSONObject payload = envelope.getJSONObject(KEY_PAYLOAD);
                    logger.trace("Replacing messaging contents: {}", payload);
                    msg.setContent(new ByteSequence(payload.toString().getBytes(StandardCharsets.UTF_8)));
                }
            }
        }

        super.send(producerExchange, msg);
    }
}
