package lc.esp.sdk;

import lc.mecha.util.BasicallyDangerous;

import java.util.concurrent.LinkedBlockingQueue;

/**
 * Implementation of an asynchronous sender for {@link ESPMessage} objects. This class
 * is useful when you are in a critical section of code (such as a game tick handler)
 * and want to send a message, but are unwilling to incur any delay.
 *
 * @author Alex Leigh
 * @since mk18 (GIPSY DANGER)
 */
public class AsyncSender extends BasicallyDangerous {
    private final ESPProducer producer;
    private final LinkedBlockingQueue<ESPMessage> q = new LinkedBlockingQueue<>();

    public AsyncSender(ESPProducer producer) {
        this.producer = producer;
    }

    public void send(ESPMessage msg) {
        q.add(msg);
    }

    @Override
    public void runDangerously() throws Exception {
        //noinspection InfiniteLoopStatement
        while (true) {
            producer.send(q.take());
        }
    }
}
