package lc.esp.sdk;

import lc.mecha.log.MechaLogger;
import lc.mecha.log.MechaLoggerFactory;

import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageConsumer;
import javax.jms.Session;

import static java.lang.Thread.sleep;

public class ESPConsumer implements AutoCloseable {
    private static final MechaLogger logger = MechaLoggerFactory.getLogger(ESPConsumer.class);
    private final MessageConsumer consumer;

    public ESPConsumer(ESPAddress destination, Session session) throws JMSException {
        consumer = session.createConsumer(destination.toDestination(session));
    }

    public ESPMessage receive() throws InterruptedException {
        while (true) {
            try {
                Message msg = consumer.receive();
                logger.debug("Received message: {}", msg);
                return new ESPMessage(msg);
            } catch (Exception e) {
                // TODO: Demote to debug when we are ready
                logger.info("Received invalid message: {}", e);
                sleep(100);
            }
        }
    }

    @Override
    public void close() throws Exception {
        consumer.close();
    }
}
