package lc.esp.sdk;

import lc.mecha.log.MechaLogger;
import lc.mecha.log.MechaLoggerFactory;

import javax.jms.JMSException;
import javax.jms.MessageProducer;
import javax.jms.Session;
import javax.jms.TextMessage;
import java.io.Serializable;
import java.util.Map;

public class ESPProducer implements AutoCloseable {
    private static final MechaLogger logger = MechaLoggerFactory.getLogger(ESPProducer.class);
    private final MessageProducer producer;
    private final Session session;

    public ESPProducer(Session session, ESPAddress address) throws JMSException {
        this.session = session;
        producer = session.createProducer(address.toDestination(session));
    }

    public void send(ESPMessage msg) throws JMSException {
        logger.debug("Sending message... {}", msg);
        TextMessage jms = session.createTextMessage(msg.getPayload().toString());

        for (Map.Entry<String, Serializable> param : msg.getParameters().entrySet()) {
            jms.setObjectProperty(param.getKey(), param.getValue());
        }

        String correlationId = msg.getCorrelationId();
        if (correlationId != null) jms.setJMSCorrelationID(correlationId);

        ESPAddress replyTo = msg.getReplyTo();
        if (replyTo != null) {
            jms.setJMSReplyTo(replyTo.toDestination(session));
        }

        producer.send(jms);
    }

    @Override
    public void close() throws Exception {
        // We leave it to ESPSession to close the session
        producer.close();
    }
}
