package lc.evelyn.svc;

import com.mchange.v2.c3p0.ComboPooledDataSource;
import lc.mecha.log.MechaLogger;
import lc.mecha.log.MechaLoggerFactory;
import org.pircbotx.PircBotX;

import java.beans.PropertyVetoException;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

public class PastebinThread implements Runnable {
    private static final MechaLogger logger = MechaLoggerFactory.getLogger(PastebinThread.class);
    private final ComboPooledDataSource cpds;
    private int lastId;
    private PircBotX bot;

    public PastebinThread(PircBotX bot) throws PropertyVetoException, SQLException {
        cpds = getDataSource();
        lastId = getPastebinMax();
        logger.info("Last pastebin id: {}", lastId);
        this.bot = bot;
    }

    private ComboPooledDataSource getDataSource() throws PropertyVetoException {
        String pw = System.getenv("LC_PASSWORD");
        ComboPooledDataSource cpds = new ComboPooledDataSource();
        cpds.setDriverClass("com.mysql.cj.jdbc.Driver");
        cpds.setJdbcUrl("jdbc:mysql://db1.leigh-co.com/pastebin");
        cpds.setUser("pastebin");
        cpds.setPassword(pw);
        return cpds;
    }

    private int getPastebinMax() throws SQLException {
        try (Connection con = cpds.getConnection()) {
            try (PreparedStatement ps = con.prepareStatement("SELECT max(id) FROM files")) {
                try (ResultSet rs = ps.executeQuery()) {
                    rs.next();
                    return rs.getInt(1);
                }
            }
        }
    }

    public void announcePastebin() throws SQLException {
        try (Connection con = cpds.getConnection()) {
            try (PreparedStatement ps = con.prepareStatement("SELECT filename FROM files WHERE id > ?")) {
                ps.setInt(1, lastId);
                try (ResultSet rs = ps.executeQuery()) {
                    while (rs.next()) {
                        bot.send().message("#leighco", "New pastebin: https://leigh-co.com/pb/obj/" + rs.getString(1));
                    }
                }
            }
        }
    }

    @Override
    public void run() {
        //noinspection InfiniteLoopStatement
        while (true) {
            try {
                int max = getPastebinMax();
                if (max != lastId) {
                    announcePastebin();
                }
                lastId = max;
            } catch (SQLException e) {
                e.printStackTrace();
            }

            try {
                Thread.sleep(1000);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        }
    }
}
