package lc.gdn.ipbeacon.service;

import lc.esp.sdk.ESPAddress;
import lc.esp.sdk.ESPClient;
import lc.esp.sdk.ESPProducer;
import lc.esp.sdk.telemetry.TelemetryFrame;
import lc.esp.sdk.telemetry.TelemetrySymbol;
import lc.mecha.http.SimpleHttpClient;
import lc.mecha.json.JSONObject;
import lc.mecha.log.MechaLogger;
import lc.mecha.log.MechaLoggerFactory;
import lc.mecha.util.BasicallyDangerous;
import lc.mecha.util.UniversalJob;
import org.apache.commons.lang.StringUtils;

import java.io.Serializable;
import java.security.KeyManagementException;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.time.Instant;
import java.util.Locale;

import static java.lang.Thread.sleep;

/**
 * Determine the public ip for this installation and periodically broadcast it to ESP.
 * <p>
 * Environment variables:
 * <p>
 * LC_SITE: The site-name for this beacon.
 *
 * @author Alex Leigh
 */
public class IPBeaconService extends BasicallyDangerous {
    public static final String ENV_SITE = "LC_EST";
    private static final MechaLogger logger = MechaLoggerFactory.getLogger(IPBeaconService.class);
    private final ESPClient esp = new ESPClient();
    private final SimpleHttpClient http = new SimpleHttpClient();
    private String est;

    public IPBeaconService() throws KeyStoreException, NoSuchAlgorithmException, KeyManagementException {
        est = System.getenv(ENV_SITE);
        if (StringUtils.isEmpty(est)) {
            logger.error(ENV_SITE + " must be set.");
            System.exit(UniversalJob.RET_BADARGS);
        }
        est = est.toUpperCase(Locale.ROOT);
    }

    public static void main(String[] args) throws Exception {
        IPBeaconService beacon = new IPBeaconService();
        beacon.run();
    }

    @Override
    public void runDangerously() throws Exception {
        esp.start();
        ESPAddress addr = new ESPAddress("topic://gdn.global.sys.mon.tlm");
        ESPProducer prod = esp.createSession().createProducer(addr);

        //noinspection InfiniteLoopStatement
        while (true) {
            JSONObject resJson = http.getJson("https://api.my-ip.io/ip.json");
            logger.info("Found IP. {}", resJson.get("ip"));

            TelemetryFrame frame = new TelemetryFrame(Instant.now());
            frame.getTags().put("est", est);
            frame.getTags().put("type", "vpn");
            frame.getSymbols().add(new TelemetrySymbol("ip.public", (Serializable) resJson.get("ip"), true));
            logger.info("Created frame: {}", frame);
            prod.send(frame.toMessage());

            sleep(60 * 10 * 1000);
        }
    }
}
