package lc.telemetry.faketsdb.service;

import lc.mecha.fabric.LiteralMessageSubscription;
import lc.mecha.http.server.WebPipeline;
import lc.mecha.http.server.WebServer;
import lc.mecha.log.MechaLogger;
import lc.mecha.log.MechaLoggerFactory;
import lc.mecha.util.BasicallyDangerous;

import java.util.HashSet;

/**
 * This class implements a fake OpenTSDB service for the purpose of receiving data from OpenTSDB clients such as
 * TCollectors. Instead of storing the data it is reformatted and sent into ESP.
 *
 * @author Alex Leigh
 * @since mk1 (HORIZON BRAVE)
 */
public class FakeTSDBService extends BasicallyDangerous {
    private static final MechaLogger logger = MechaLoggerFactory.getLogger(FakeTSDBService.class);

    public static void main(String[] args) throws Exception {
        FakeTSDBService svc = new FakeTSDBService();
        svc.runDangerously();
    }

    @Override
    public void runDangerously() throws Exception {
        // 4242 is the default OpenTSDB port
        WebServer srv = new WebServer(4242, 128);
        HashSet<String> prefixes = new HashSet<>();
        prefixes.add("/api");

        srv.getWebPipeline().getServicePipeline().getSubscriptionBase().subscribe(
                new LiteralMessageSubscription<>(new TSDBRelay(prefixes), WebPipeline.KEY_SERVICE));

        srv.start();
    }
}
