package lc.zero.sdk;

import lc.mecha.json.JSONObject;
import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;

import java.io.IOException;
import java.util.HashMap;

/**
 * This class holds a ZERO activation index.
 *
 * @author Alex Leigh
 * @since mk1 (PUMA REAL)
 */
public class ZeroActivationIndex {
    public static final String KEY_SERVICES = "services";
    private final HashMap<String, String> services = new HashMap<>();
    private final HttpClient httpClient;

    public ZeroActivationIndex(HttpClient httpClient, JSONObject obj) {
        this.httpClient = httpClient;
        JSONObject svcs = obj.getJSONObject(KEY_SERVICES);
        for (String svcName : svcs.keySet()) {
            String cfgUrl = svcs.getString(svcName);
            services.put(svcName, cfgUrl);
        }
    }

    public ZeroServiceConfig readConfig(String service) throws IOException {
        String svcUrl = services.get(service);
        if (svcUrl == null) return null;

        HttpResponse response = httpClient.execute(new HttpGet(svcUrl));
        HttpEntity entity = response.getEntity();
        JSONObject json = new JSONObject(entity.getContent());
        return new ZeroServiceConfig(service, json);
    }

    @Override
    public String toString() {
        return "ZeroActivationIndex{" +
                "services=" + services +
                '}';
    }
}
